@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: error.c,v 1.15 2004/07/18 20:44:01 ats Exp $
+ * $Id: error.c,v 1.16 2004/11/01 18:30:17 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -128,6 +128,8 @@ const char *sg_str_error(sg_error code) {
 		return "unsupported function";
 	case SG_ERROR_XSW_VER_MISMATCH:
 		return "XSW version mismatch";
+	case SG_ERROR_PSTAT:
+		return "pstat failed";
 	}
 	return "unknown error";
 }
