@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: load_stats.c,v 1.15 2004/04/07 21:08:40 tdb Exp $ 
+ * $Id: load_stats.c,v 1.17 2004/11/07 12:29:20 ats Exp $ 
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,12 +35,20 @@
 #include <kstat.h>
 #endif
 #endif
+#ifdef HPUX
+#include <sys/param.h>
+#include <sys/pstat.h>
+#endif
 
 sg_load_stats *sg_get_load_stats(){
 
 	static sg_load_stats load_stat;
 
+#ifdef HPUX
+	struct pst_dynamic pstat_dynamic;
+#else
 	double loadav[3];
+#endif
 
 #ifdef CYGWIN
 	sg_set_error(SG_ERROR_UNSUPPORTED, "Cygwin");
@@ -85,6 +93,15 @@ sg_load_stats *sg_get_load_stats(){
 		return NULL;
 	}
 	load_stat.min15 = (double)kn->value.ui32 / (double)256;
+#elif defined(HPUX)
+	if (pstat_getdynamic(&pstat_dynamic, sizeof(pstat_dynamic), 1, 0) == -1) {
+		sg_set_error_with_errno(SG_ERROR_PSTAT, "pstat_dynamic");
+		return NULL;
+	}
+
+	load_stat.min1=pstat_dynamic.psd_avg_1_min;
+	load_stat.min5=pstat_dynamic.psd_avg_5_min;
+	load_stat.min15=pstat_dynamic.psd_avg_15_min;
 #else
 	getloadavg(loadav,3);
 
