@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: network_stats.c,v 1.71 2004/11/08 08:07:25 ats Exp $
+ * $Id: network_stats.c,v 1.72 2005/02/24 18:35:11 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -675,10 +675,10 @@ sg_network_iface_stats *sg_get_network_iface_stats(int *entries){
 			network_iface_stat_ptr->speed = ethcmd.speed;
 
 			switch (ethcmd.duplex) {
-			case 0x00:
+			case DUPLEX_FULL:
 				network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_FULL;
 				break;
-			case 0x01:
+			case DUPLEX_HALF:
 				network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_HALF;
 				break;
 			default:
