@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: process_stats.c,v 1.74 2004/07/18 21:30:11 ats Exp $
+ * $Id: process_stats.c,v 1.80 2004/11/07 12:32:24 ats Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,8 +33,8 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <dirent.h>
-#include <string.h>
 #endif
+#include <string.h>
 
 #ifdef SOLARIS
 #include <procfs.h>
@@ -72,6 +72,12 @@
 #include <sys/lwp.h>
 #endif
 #endif
+#ifdef HPUX
+#include <sys/param.h>
+#include <sys/pstat.h>
+#include <unistd.h>
+#define PROCESS_BATCH 30
+#endif
 
 static void proc_state_init(sg_process_stats *s) {
 	s->process_name = NULL;
@@ -88,6 +94,12 @@ sg_process_stats *sg_get_process_stats(int *entries){
 			      proc_state_init, proc_state_destroy);
 	int proc_state_size = 0;
 	sg_process_stats *proc_state_ptr;
+#ifdef HPUX
+	struct pst_status pstat_procinfo[PROCESS_BATCH];
+	long procidx = 0;
+	long long pagesize;
+	int num, i;
+#endif
 #ifdef ALLBSD
 	int mib[4];
 	size_t size;
@@ -192,21 +204,41 @@ sg_process_stats *sg_get_process_stats(int *entries){
 			return NULL;
 		}
 
-		if(process_info.pr_lwp.pr_state==1) proc_state_ptr->state = SG_PROCESS_STATE_SLEEPING;
-		if(process_info.pr_lwp.pr_state==2) proc_state_ptr->state = SG_PROCESS_STATE_RUNNING; 
-		if(process_info.pr_lwp.pr_state==3) proc_state_ptr->state = SG_PROCESS_STATE_ZOMBIE; 
-		if(process_info.pr_lwp.pr_state==4) proc_state_ptr->state = SG_PROCESS_STATE_STOPPED; 
-		if(process_info.pr_lwp.pr_state==6) proc_state_ptr->state = SG_PROCESS_STATE_RUNNING; 
+		switch (process_info.pr_lwp.pr_state) {
+		case 1:
+			proc_state_ptr->state = SG_PROCESS_STATE_SLEEPING;
+			break;
+		case 2:
+		case 5:
+			proc_state_ptr->state = SG_PROCESS_STATE_RUNNING; 
+			break;
+		case 3:
+			proc_state_ptr->state = SG_PROCESS_STATE_ZOMBIE; 
+			break;
+		case 4:
+			proc_state_ptr->state = SG_PROCESS_STATE_STOPPED; 
+			break;
+		}
 #endif
 #ifdef LINUX
 		x = fscanf(f, "%d %4096s %c %d %d %*d %*d %*d %*u %*u %*u %*u %*u %lu %lu %*d %*d %*d %d %*d %*d %lu %llu %llu %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*d %*d\n", &(proc_state_ptr->pid), ps_name, &s, &(proc_state_ptr->parent), &(proc_state_ptr->pgid), &utime, &stime, &(proc_state_ptr->nice), &starttime, &(proc_state_ptr->proc_size), &(proc_state_ptr->proc_resident));
 		/* +3 becuase man page says "Resident  Set Size: number of pages the process has in real memory, minus 3 for administrative purposes." */
 		proc_state_ptr->proc_resident = (proc_state_ptr->proc_resident + 3) * getpagesize();
-		if(s == 'S') proc_state_ptr->state = SG_PROCESS_STATE_SLEEPING;
-		if(s == 'R') proc_state_ptr->state = SG_PROCESS_STATE_RUNNING;
-		if(s == 'Z') proc_state_ptr->state = SG_PROCESS_STATE_ZOMBIE;
-		if(s == 'T') proc_state_ptr->state = SG_PROCESS_STATE_STOPPED;
-		if(s == 'D') proc_state_ptr->state = SG_PROCESS_STATE_STOPPED;
+		switch (s) {
+		case 'S':
+			proc_state_ptr->state = SG_PROCESS_STATE_SLEEPING;
+			break;
+		case 'R':
+			proc_state_ptr->state = SG_PROCESS_STATE_RUNNING;
+			break;
+		case 'Z':
+			proc_state_ptr->state = SG_PROCESS_STATE_ZOMBIE;
+			break;
+		case 'T':
+		case 'D':
+			proc_state_ptr->state = SG_PROCESS_STATE_STOPPED;
+			break;
+		}
 	
 		/* pa_name[0] should = '(' */
 		ptr = strchr(&ps_name[1], ')');	
@@ -384,7 +416,7 @@ sg_process_stats *sg_get_process_stats(int *entries){
 		proctitle = NULL;
 
 		do {
-			if(size >= buflen) {
+			if((long) size >= buflen) {
 				buflen *= 2;
 				size = buflen;
 				proctitletmp = sg_realloc(proctitle, buflen);
@@ -406,7 +438,7 @@ sg_process_stats *sg_get_process_stats(int *entries){
 				size = 0;
 				break;
 			}
-		} while(size >= buflen);
+		} while((long) size >= buflen);
 
 		if(size > 0) {
 			proc_state_ptr->proctitle = sg_malloc(size+1);
@@ -647,6 +679,78 @@ sg_process_stats *sg_get_process_stats(int *entries){
 	free(kp_stats);
 #endif
 
+#ifdef HPUX
+	if ((pagesize = sysconf(_SC_PAGESIZE)) == -1) {
+		sg_set_error_with_errno(SG_ERROR_SYSCONF, "_SC_PAGESIZE");
+		return NULL;
+	}
+
+	while (1) {
+		num = pstat_getproc(pstat_procinfo, sizeof pstat_procinfo[0],
+		                    PROCESS_BATCH, procidx);
+		if (num == -1) {
+			sg_set_error_with_errno(SG_ERROR_PSTAT,
+			                        "pstat_getproc");
+			return NULL;
+		} else if (num == 0) {
+			break;
+		}
+
+		for (i = 0; i < num; i++) {
+			struct pst_status *pi = &pstat_procinfo[i];
+
+			if (VECTOR_RESIZE(proc_state, proc_state_size + 1) < 0) {
+				return NULL;
+			}
+			proc_state_ptr = proc_state+proc_state_size;
+	
+			proc_state_ptr->pid = pi->pst_pid;
+			proc_state_ptr->parent = pi->pst_ppid;
+			proc_state_ptr->pgid = pi->pst_pgrp;
+			proc_state_ptr->uid = pi->pst_uid;
+			proc_state_ptr->euid = pi->pst_euid;
+			proc_state_ptr->gid = pi->pst_gid;
+			proc_state_ptr->egid = pi->pst_egid;
+			proc_state_ptr->proc_size = (pi->pst_dsize + pi->pst_tsize + pi->pst_ssize) * pagesize;
+			proc_state_ptr->proc_resident = pi->pst_rssize * pagesize;
+			proc_state_ptr->time_spent = pi->pst_time;
+			proc_state_ptr->cpu_percent = (pi->pst_pctcpu * 100.0) / 0x8000;
+			proc_state_ptr->nice = pi->pst_nice;
+	
+			if (sg_update_string(&proc_state_ptr->process_name,
+					     pi->pst_ucomm) < 0) {
+				return NULL;
+			}
+			if (sg_update_string(&proc_state_ptr->proctitle,
+					     pi->pst_cmd) < 0) {
+				return NULL;
+			}
+	
+			switch (pi->pst_stat) {
+			case PS_SLEEP:
+				proc_state_ptr->state = SG_PROCESS_STATE_SLEEPING;
+				break;
+			case PS_RUN:
+				proc_state_ptr->state = SG_PROCESS_STATE_RUNNING;
+				break;
+			case PS_STOP:
+				proc_state_ptr->state = SG_PROCESS_STATE_STOPPED;
+				break;
+			case PS_ZOMBIE:
+				proc_state_ptr->state = SG_PROCESS_STATE_ZOMBIE;
+				break;
+			case PS_IDLE:
+			case PS_OTHER:
+				proc_state_ptr->state = SG_PROCESS_STATE_UNKNOWN;
+				break;
+			}
+	
+			proc_state_size++;
+		}
+		procidx = pstat_procinfo[num - 1].pst_idx + 1;
+	}
+#endif
+
 #ifdef CYGWIN
 	sg_set_error(SG_ERROR_UNSUPPORTED, "Cygwin");
 	return NULL;
