@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: statgrab.h,v 1.53 2004/07/18 20:44:01 ats Exp $
+ * $Id: statgrab.h,v 1.54 2004/11/01 18:30:17 tdb Exp $
  */
 
 #ifndef STATGRAB_H
@@ -63,7 +63,8 @@ typedef enum {
 	SG_ERROR_SYSCTLNAMETOMIB,
 	SG_ERROR_UNAME,
 	SG_ERROR_UNSUPPORTED,
-	SG_ERROR_XSW_VER_MISMATCH
+	SG_ERROR_XSW_VER_MISMATCH,
+	SG_ERROR_PSTAT
 } sg_error;
 
 void sg_set_error(sg_error code, const char *arg);
